import os
from parsimonious import Grammar, NodeVisitor


# cache
_grammar = None


def load_grammar():
    global _grammar
    parser_dir = os.path.dirname(__file__)
    with open(os.path.join(parser_dir, 'grammar.ebnf'), 'rt') as grammar_file:
        grammar_text = grammar_file.read()
    _grammar = Grammar(grammar_text)


def parse(input):
    if _grammar is None:
        load_grammar()
    return _grammar.parse(input)


def cleanup_children(visited_children):
    children = [c for c in visited_children if c]
    if len(children) == 1:
        return children[0]
    else:
        return children


class Compiler(NodeVisitor):
    def generic_visit(self, node, visited_children):
        return cleanup_children(visited_children)

    def visit_or_expression(self, node, visited_children):
        return ('OR', visited_children[0], visited_children[2])

    def visit_and_expression(self, node, visited_children):
        return ('AND', visited_children[0], visited_children[2])

    def visit_not_expression(self, node, visited_children):
        return ('NOT', visited_children[2])

    def visit_parens_expression(self, node, visited_children):
        return ('PARENS', visited_children[2])

    def visit_name(self, node, visited_children):
        return ('NAME', node.text)


def compile(tree):
    if isinstance(tree, str):
        tree = parse(tree)
    return Compiler().visit(tree)
