#! /usr/bin/env python

import unittest
from parsimonious import ParseError
from parser import compile

class TestParser(unittest.TestCase):
    def _parse(self, input):
        return compile(input)

    def test_02_tag(self):
        self.assertEqual(self._parse('xxx'), ('NAME', 'xxx'))

    def test_03_bad_tag(self):
        self.assertRaises(ParseError, self._parse, 'XXX')

    def test_04_expression(self):
        self.assertEqual(self._parse('!(xxx&yyy)'),
            ('NOT', ('PARENS', ('AND', ('NAME', 'xxx'), ('NAME', 'yyy'))))
        )
        self.assertEqual(self._parse('!(xxx & yyy)'),
            ('NOT', ('PARENS', ('AND', ('NAME', 'xxx'), ('NAME', 'yyy'))))
        )
        self.assertEqual(self._parse('!xxx&yyy&zzz|ooo'),
            ('OR', ('AND',
                ('NOT', ('NAME', 'xxx')),
                ('AND', ('NAME', 'yyy'), ('NAME', 'zzz'))),
            ('NAME', 'ooo'))
        )
        self.assertEqual(self._parse('!(xxx &&  yyy)'),
            ('NOT', ('PARENS', ('AND', ('NAME', 'xxx'), ('NAME', 'yyy'))))
        )
        self.assertEqual(self._parse('!(xxx || yyy)'),
            ('NOT', ('PARENS', ('OR', ('NAME', 'xxx'), ('NAME', 'yyy'))))
        )
        self.assertEqual(self._parse('xxx and yyy'),
            ('AND', ('NAME', 'xxx'), ('NAME', 'yyy'))
        )
        self.assertEqual(self._parse('xxx or  yyy'),
            ('OR', ('NAME', 'xxx'), ('NAME', 'yyy'))
        )
        self.assertEqual(self._parse('xxx OR yyy'),
            ('OR', ('NAME', 'xxx'), ('NAME', 'yyy'))
        )
        self.assertEqual(self._parse('not xxx'),
            ('NOT', ('NAME', 'xxx'))
        )
        self.assertEqual(self._parse('NOT xxx'),
            ('NOT', ('NAME', 'xxx'))
        )
        self.assertEqual(self._parse('NOT (xxx & yyy) AND zzz | ooo'),
            ('OR',
                ('AND',
                    ('NOT',
                        ('PARENS',
                            ('AND', ('NAME', 'xxx'), ('NAME', 'yyy'))
                        )
                    ),
                    ('NAME', 'zzz')
                ),
                ('NAME', 'ooo')
            )
        )

    def test_05_bad_expression(self):
        self.assertRaises(ParseError, self._parse, '!(xxx&yyy')

if __name__ == "__main__":
    unittest.main()
